*----------------------------------------------------------------------------
* Graphics functions
* VGA 640*480*8bit mode, FPGA-based framebuffer with hardware acceleration 
*----------------------------------------------------------------------------

*  pppp pppp pppp pppp pppp pppp pppp pppp RRRR
*   +32  +28  +24  +20  +16  +12   +8   +4   sp

                .include "gfx_asm.i"
*----------------------------------------------------------------------------
                .macro gfx_wait
                btst    #7,GFX_COMMAND_STATUS_REGISTER
                bne.s  .-6
                .endm
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_load_LED_register
                .type   GFX_load_LED_register, @function
GFX_load_LED_register:
                move.b  7(%sp),TEST_LED_REGISTER
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  _video_IRQ7_handler

_video_IRQ7_handler:
                rte
*----------------------------------------------------------------------------

                .section .text
                .align  2
                .globl  GFX_initialize
                .type   GFX_initialize, @function
GFX_initialize:
                moveq   #-1,%d0
                move.l  %d0,GFX_BIT_MASK_REGISTER
                move.b  #0x0F,RAMDAC_PIXEL_MASK_REGISTER
                lea     _gfx_sys_pal0,%a0
                clr.b   %d0
                move.w  #48,%d1 
                bsr.s   _gfx_wr_pal
                bra.s   _gfx_cls
_gfx_sys_pal0:
                dc.l    0x00000000,  0x001F001F,  0x00001F1F,  0x1F00001F
                dc.l    0x001F1F1F,  0x001F1F1F,  0x0F0F0F00,  0x003F003F
                dc.l    0x00003F3F,  0x3F00003F,  0x003F3F3F,  0x003F3F3F
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_RAMDAC_set_RGB_palette
                .type   GFX_RAMDAC_set_RGB_palette, @function

GFX_RAMDAC_set_RGB_palette:

                move.w  14(%sp),%d1
                beq.s   _gfx_setrgbpal_exit
                cmpi.w  #256,%d1
                bhi.s   _gfx_setrgbpal_exit
                lsl.w   #1,%d1
                add.w   14(%sp),%d1
                movea.l 4(%sp),%a0
                move.b  11(%sp),%d0
_gfx_wr_pal:    
                subq.w  #1,%d1
                move.b  %d0,RAMDAC_PALETTE_INDEX_REGISTER
_gfx_wr_pal_loop:
                move.b  (%a0)+,RAMDAC_PALETTE_RAM_REGISTER
                dbra    %d1,_gfx_wr_pal_loop
_gfx_setrgbpal_exit:
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_RAMDAC_set_pixel_mask
                .type   GFX_RAMDAC_set_pixel_mask, @function

GFX_RAMDAC_set_pixel_mask:
                move.b  7(%sp),RAMDAC_PIXEL_MASK_REGISTER
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_clear_screen
                .type   GFX_clear_screen, @function

GFX_clear_screen:
                move.b  7(%sp),%d0
_gfx_cls:
                gfx_wait
                move.b  %d0,GFX_COLOR_REGISTER
                move.w  #320,GFX_LENGTH_REGISTER
                clr.w   GFX_DESTINATION_X_REGISTER
                move.w  #479,%d0
_gfx_cls_y_loop:
                gfx_wait
                move.w  %d0,GFX_DESTINATION_Y_REGISTER
                move.b  #0xBF,GFX_COMMAND_STATUS_REGISTER
                dbra    %d0,_gfx_cls_y_loop
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_set_draw_mask
                .type   GFX_set_draw_mask, @function
GFX_set_draw_mask:
                gfx_wait
                move.l  4(%sp),GFX_BIT_MASK_REGISTER
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_line
                .type   GFX_line, @function

*  c... y2.. x2.. y1.. x1.. RRRR
*  +23  +18  +14  +10  +6   sp
GFX_line:
                gfx_wait
                move.b  23(%sp),GFX_COLOR_REGISTER
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.w  14(%sp),GFX_X2_REGISTER 
                move.w  18(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,GFX_COMMAND_STATUS_REGISTER
                rts
*----------------------------------------------------------------------------
                .section .text
                .align  2
                .globl  GFX_rectangle
                .type   GFX_rectangle, @function

GFX_rectangle:
*  c... y2.. x2.. y1.. x1.. RRRR
*  +23  +18  +14  +10  +6   sp

                movea.l #GFX_COMMAND_STATUS_REGISTER,%a0
                gfx_wait
                move.b  23(%sp),GFX_COLOR_REGISTER
* top horizontal line
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  14(%sp),GFX_X2_REGISTER 
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.w  10(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,(%a0)
                gfx_wait
*right vertical line
                move.w  18(%sp),GFX_Y1_REGISTER 
                move.w  14(%sp),GFX_X1_REGISTER
                move.b  #0xB8,(%a0)
                gfx_wait
*bottom horizontal line
                move.w  6(%sp),GFX_X2_REGISTER 
                move.w  18(%sp),GFX_Y2_REGISTER
                move.b  #0xB8,(%a0)
                gfx_wait
                move.w  6(%sp),GFX_X1_REGISTER
                move.w  10(%sp),GFX_Y1_REGISTER 
                move.b  #0xB8,(%a0)
                rts
*----------------------------------------------------------------------------
